@@ -28,16 +28,17 @@ __docformat__ = "restructuredtext en"
 
 class Interface(object):
     """Base class for interfaces."""
+
     @classmethod
-    def is_implemented_by(cls, instance):
+    def is_implemented_by(cls, instance: type) -> bool:
         return implements(instance, cls)
 
 
-def implements(obj, interface):
+def implements(obj: type, interface: type) -> bool:
     """Return true if the give object (maybe an instance or class) implements
     the interface.
     """
-    kimplements = getattr(obj, '__implements__', ())
+    kimplements = getattr(obj, "__implements__", ())
     if not isinstance(kimplements, (list, tuple)):
         kimplements = (kimplements,)
     for implementedinterface in kimplements:
@@ -46,7 +47,7 @@ def implements(obj, interface):
     return False
 
 
-def extend(klass, interface, _recurs=False):
+def extend(klass: type, interface: type, _recurs: bool = False) -> None:
     """Add interface to klass'__implements__ if not already implemented in.
 
     If klass is subclassed, ensure subclasses __implements__ it as well.
@@ -55,14 +56,14 @@ def extend(klass, interface, _recurs=False):
     """
     if not implements(klass, interface):
         try:
-            kimplements = klass.__implements__
+            kimplements = klass.__implements__  # type: ignore
             kimplementsklass = type(kimplements)
             kimplements = list(kimplements)
         except AttributeError:
             kimplementsklass = tuple
             kimplements = []
         kimplements.append(interface)
-        klass.__implements__ = kimplementsklass(kimplements)
+        klass.__implements__ = kimplementsklass(kimplements)  # type: ignore
         for subklass in klass.__subclasses__():
             extend(subklass, interface, _recurs=True)
     elif _recurs:
