@@ -24,8 +24,10 @@ from encodings import search_function
 import sys
 from email.utils import parseaddr, parsedate
 from email.header import decode_header
+from email.message import Message
 
 from datetime import datetime
+from typing import Any, Optional, List, Tuple, Union
 
 try:
     from mx.DateTime import DateTime
@@ -35,48 +37,54 @@ except ImportError:
 import logilab.common as lgc
 
 
-def decode_QP(string):
-    parts = []
-    for decoded, charset in decode_header(string):
-        if not charset :
-            charset = 'iso-8859-15'
+def decode_QP(string: str) -> str:
+    parts: List[str] = []
+    for maybe_decoded, charset in decode_header(string):
+        if not charset:
+            charset = "iso-8859-15"
         # python 3 sometimes returns str and sometimes bytes.
         # the 'official' fix is to use the new 'policy' APIs
         # https://bugs.python.org/issue24797
         # let's just handle this bug ourselves for now
-        if isinstance(decoded, bytes):
-            decoded = decoded.decode(charset, 'replace')
+        if isinstance(maybe_decoded, bytes):
+            decoded = maybe_decoded.decode(charset, "replace")
+        else:
+            decoded = maybe_decoded
+
         assert isinstance(decoded, str)
         parts.append(decoded)
 
     if sys.version_info < (3, 3):
         # decoding was non-RFC compliant wrt to whitespace handling
         # see http://bugs.python.org/issue1079
-        return u' '.join(parts)
-    return u''.join(parts)
+        return " ".join(parts)
+
+    return "".join(parts)
+
 
 def message_from_file(fd):
     try:
         return UMessage(email.message_from_file(fd))
     except email.errors.MessageParseError:
-        return ''
+        return ""
 
-def message_from_string(string):
+
+def message_from_string(string: str) -> Union["UMessage", str]:
     try:
         return UMessage(email.message_from_string(string))
     except email.errors.MessageParseError:
-        return ''
+        return ""
+
 
 class UMessage:
-    """Encapsulates an email.Message instance and returns only unicode objects.
-    """
+    """Encapsulates an email.Message instance and returns only unicode objects."""
 
-    def __init__(self, message):
+    def __init__(self, message: Message) -> None:
         self.message = message
 
     # email.Message interface #################################################
 
-    def get(self, header, default=None):
+    def get(self, header: str, default: Optional[Any] = None) -> Optional[str]:
         value = self.message.get(header, default)
         if value:
             return decode_QP(value)
@@ -85,9 +93,8 @@ class UMessage:
     def __getitem__(self, header):
         return self.get(header)
 
-    def get_all(self, header, default=()):
-        return [decode_QP(val) for val in self.message.get_all(header, default)
-                if val is not None]
+    def get_all(self, header: str, default: Tuple[()] = ()) -> List[str]:
+        return [decode_QP(val) for val in self.message.get_all(header, default) if val is not None]
 
     def is_multipart(self):
         return self.message.is_multipart()
@@ -99,23 +106,36 @@ class UMessage:
         for part in self.message.walk():
             yield UMessage(part)
 
-    def get_payload(self, index=None, decode=False):
+    def get_payload(
+        self, index: Optional[Any] = None, decode: bool = False
+    ) -> Union[str, "UMessage", List["UMessage"]]:
         message = self.message
+
         if index is None:
-            payload = message.get_payload(index, decode)
+            # mypy: Argument 1 to "get_payload" of "Message" has incompatible type "None";
+            # mypy: expected "int"
+            # email.message.Message.get_payload has type signature:
+            # Message.get_payload(self, i=None, decode=False)
+            # so None seems to be totally acceptable, I don't understand mypy here
+            payload = message.get_payload(index, decode)  # type: ignore
+
             if isinstance(payload, list):
                 return [UMessage(msg) for msg in payload]
-            if message.get_content_maintype() != 'text':
+
+            if message.get_content_maintype() != "text":
                 return payload
+
             if isinstance(payload, str):
                 return payload
 
-            charset = message.get_content_charset() or 'iso-8859-1'
+            charset = message.get_content_charset() or "iso-8859-1"
             if search_function(charset) is None:
-                charset = 'iso-8859-1'
-            return str(payload or b'', charset, "replace")
+                charset = "iso-8859-1"
+
+            return str(payload or b"", charset, "replace")
         else:
             payload = UMessage(message.get_payload(index, decode))
+
         return payload
 
     def get_content_maintype(self):
@@ -131,7 +151,7 @@ class UMessage:
         try:
             return str(value)
         except UnicodeDecodeError:
-            return u'error decoding filename'
+            return "error decoding filename"
 
     # other convenience methods ###############################################
 
@@ -139,8 +159,8 @@ class UMessage:
         """return an unicode string containing all the message's headers"""
         values = []
         for header in self.message.keys():
-            values.append(u'%s: %s' % (header, self.get(header)))
-        return '\n'.join(values)
+            values.append("%s: %s" % (header, self.get(header)))
+        return "\n".join(values)
 
     def multi_addrs(self, header):
         """return a list of 2-uple (name, address) for the given address (which
@@ -156,7 +176,7 @@ class UMessage:
         """return a datetime object for the email's date or None if no date is
         set or if it can't be parsed
         """
-        value = self.get('date')
+        value = self.get("date")
         if value is None and alternative_source:
             unix_from = self.message.get_unixfrom()
             if unix_from is not None:
