@@ -18,49 +18,60 @@
 """Text formatting drivers for ureports"""
 
 from __future__ import print_function
+from typing import Any, List, Tuple
 
 __docformat__ = "restructuredtext en"
 
 from logilab.common.textutils import linesep
 from logilab.common.ureports import BaseWriter
+from logilab.common.ureports.nodes import (
+    Section,
+    Title,
+    Table,
+    List as NodeList,
+    Paragraph,
+    Link,
+    VerbatimText,
+    Text,
+)
 
 
-TITLE_UNDERLINES = [u'', u'=', u'-', u'`', u'.', u'~', u'^']
-BULLETS = [u'*', u'-']
+TITLE_UNDERLINES = ["", "=", "-", "`", ".", "~", "^"]
+BULLETS = ["*", "-"]
 
 
 class TextWriter(BaseWriter):
     """format layouts as text
     (ReStructured inspiration but not totally handled yet)
     """
-    def begin_format(self, layout):
+
+    def begin_format(self, layout: Any) -> None:
         super(TextWriter, self).begin_format(layout)
         self.list_level = 0
-        self.pending_urls = []
+        self.pending_urls: List[Tuple[str, str]] = []
 
-    def visit_section(self, layout):
-        """display a section as text
-        """
+    def visit_section(self, layout: Section) -> None:
+        """display a section as text"""
         self.section += 1
         self.writeln()
         self.format_children(layout)
         if self.pending_urls:
             self.writeln()
             for label, url in self.pending_urls:
-                self.writeln(u'.. _`%s`: %s' % (label, url))
+                self.writeln(".. _`%s`: %s" % (label, url))
             self.pending_urls = []
         self.section -= 1
         self.writeln()
 
-    def visit_title(self, layout):
-        title = u''.join(list(self.compute_content(layout)))
+    def visit_title(self, layout: Title) -> None:
+        title = "".join(list(self.compute_content(layout)))
         self.writeln(title)
         try:
             self.writeln(TITLE_UNDERLINES[self.section] * len(title))
         except IndexError:
             print("FIXME TITLE TOO DEEP. TURNING TITLE INTO TEXT")
 
-    def visit_paragraph(self, layout):
+    def visit_paragraph(self, layout: "Paragraph") -> None:
         """enter a paragraph"""
         self.format_children(layout)
         self.writeln()
@@ -69,76 +80,82 @@ class TextWriter(BaseWriter):
         """enter a span"""
         self.format_children(layout)
 
-    def visit_table(self, layout):
+    def visit_table(self, layout: Table) -> None:
         """display a table as text"""
         table_content = self.get_table_content(layout)
         # get columns width
-        cols_width = [0]*len(table_content[0])
+        cols_width = [0] * len(table_content[0])
         for row in table_content:
             for index in range(len(row)):
                 col = row[index]
                 cols_width[index] = max(cols_width[index], len(col))
-        if layout.klass == 'field':
+        if layout.klass == "field":
             self.field_table(layout, table_content, cols_width)
         else:
             self.default_table(layout, table_content, cols_width)
         self.writeln()
 
-    def default_table(self, layout, table_content, cols_width):
+    def default_table(
+        self, layout: Table, table_content: List[List[str]], cols_width: List[int]
+    ) -> None:
         """format a table"""
-        cols_width = [size+1 for size in cols_width]
-        format_strings = u' '.join([u'%%-%ss'] * len(cols_width))
+        cols_width = [size + 1 for size in cols_width]
+
+        format_strings = " ".join(["%%-%ss"] * len(cols_width))
         format_strings = format_strings % tuple(cols_width)
-        format_strings = format_strings.split(' ')
-        table_linesep = (
-            u'\n+' + u'+'.join([u'-'*w for w in cols_width]) + u'+\n')
-        headsep = u'\n+' + u'+'.join([u'='*w for w in cols_width]) + u'+\n'
+
+        format_strings_list = format_strings.split(" ")
+
+        table_linesep = "\n+" + "+".join(["-" * w for w in cols_width]) + "+\n"
+        headsep = "\n+" + "+".join(["=" * w for w in cols_width]) + "+\n"
+
         # FIXME: layout.cheaders
         self.write(table_linesep)
         for i in range(len(table_content)):
-            self.write(u'|')
+            self.write("|")
             line = table_content[i]
             for j in range(len(line)):
-                self.write(format_strings[j] % line[j])
-                self.write(u'|')
+                self.write(format_strings_list[j] % line[j])
+                self.write("|")
             if i == 0 and layout.rheaders:
                 self.write(headsep)
             else:
                 self.write(table_linesep)
 
-    def field_table(self, layout, table_content, cols_width):
+    def field_table(
+        self, layout: Table, table_content: List[List[str]], cols_width: List[int]
+    ) -> None:
         """special case for field table"""
         assert layout.cols == 2
-        format_string = u'%s%%-%ss: %%s' % (linesep, cols_width[0])
+        format_string = "%s%%-%ss: %%s" % (linesep, cols_width[0])
         for field, value in table_content:
             self.write(format_string % (field, value))
 
-    def visit_list(self, layout):
+    def visit_list(self, layout: NodeList) -> None:
         """display a list layout as text"""
         bullet = BULLETS[self.list_level % len(BULLETS)]
-        indent = '  ' * self.list_level
+        indent = "  " * self.list_level
         self.list_level += 1
         for child in layout.children:
-            self.write(u'%s%s%s ' % (linesep, indent, bullet))
+            self.write("%s%s%s " % (linesep, indent, bullet))
             child.accept(self)
         self.list_level -= 1
 
-    def visit_link(self, layout):
+    def visit_link(self, layout: Link) -> None:
         """add a hyperlink"""
         if layout.label != layout.url:
-            self.write(u'`%s`_' % layout.label)
+            self.write("`%s`_" % layout.label)
             self.pending_urls.append((layout.label, layout.url))
         else:
             self.write(layout.url)
 
-    def visit_verbatimtext(self, layout):
-        """display a verbatim layout as text (so difficult ;)
-        """
-        self.writeln(u'::\n')
+    def visit_verbatimtext(self, layout: VerbatimText) -> None:
+        """display a verbatim layout as text (so difficult ;)"""
+        self.writeln("::\n")
         for line in layout.data.splitlines():
-            self.writeln(u'    ' + line)
+            self.writeln("    " + line)
         self.writeln()
 
-    def visit_text(self, layout):
+    def visit_text(self, layout: Text) -> None:
         """add some text"""
-        self.write(u'%s' % layout.data)
+        self.write("%s" % layout.data)
